<?php


// Here are some general regular expressions
$invalid_chars = '/([<|>])/';
$date_regexp_sep = '[-|\.|\/]';
$date_regexp = "/^(\d\d\d\d)$date_regexp_sep(\d\d)$date_regexp_sep(\d\d)$/";
$decimal_regexp = "/\d+\.?\d*/";
$entity_map = array(
			' '	=>	" ",
			'@'	=>	"&#064;",
			'<'	=>	"&lt;",
			'>'	=>	"&gt;",
		   );



// Here goes the validating functions

function element_valid ($node, &$namespace, $level, &$path, $type, $min, $max) {
    global $xml_header, $p, $invalid_chars, $entity_map, $debug, $f, $indent, $date_regexp, $decimal_regexp;
    $value = get_value($f, $path);
    $offset = $path[count($path) - 1];

    if ($type != "xs:anyType") {
	if (preg_match($invalid_chars, $value, $match)) {
	    set_value($p, $path, preg_replace($invalid_chars, "", $value));
	    return "Ulovligt tegn i felt: &#034;" . $entity_map[$match[1]] . "&#034;";
	}
    }

    if (!strlen(trim($value))) {			// Empty fields
	if ((string)$offset == 'attr') {
	    // This is not implemented yet, so far, atttributes are always optional...
	} elseif ($offset < $min) {
	    return "Feltet er ikke angivet";
	}
    } elseif ($type == "xs:string") {				// Various types
    } elseif ($type == "xs:date") {
	if (preg_match($date_regexp, $value, $match)) {
	    $year = $match[1];
	    $month = $match[2];
	    $day = $match[3];
	    if (($month < 1) || ($month > 12)) {
		return "Mned skal vre 1-12";
	    } elseif (($day < 1) || ($day > 31)) {
		return "Dag skal vre 1-31";
	    }
	} else {
	    return "Datoformatet er -MM-DD";
	}
    } elseif ($type == "xs:postiveInteger") {
	if (((int)$value != $value) || ($value <= 0)) {
	    return "Skal vre positivt heltal";
	}
    } elseif ($type == "xs:integer") {
	if ((int)$value != $value) {
	    return "Skal vre heltallig";
	}
    } elseif ($type == "xs:decimal") {
	if (!preg_match($decimal_regexp, $value)) {
	    return "Skal vre et decimaltal";
	}
    } elseif ($type == "xs:anyURI") {
	$fields = parse_url($value);
	if ($scheme = $fields['scheme']) {
	    if (preg_match("/([\s|\@])/", $scheme, $match)) {
		return "Metoden m ikke indeholde &#034;" . $entity_map[$match[1]] . "&#034;";
	    }
	} else {
	    return "Husk at angive metode, http, ftp, gopher etc.";
	}
	if ($host = $fields['host']) {
	    if (preg_match("/([\s|\@])/", $host, $match)) {
		return "Hostnavnet m ikke indeholde &#034;" . $entity_map[$match[1]] . "&#034;";
	    }
	} else {
	    return "Manglende information om host";
	}
	if ($doc_path = $fields['path']) {
	    if (preg_match("/([\s])/", $doc_path, $match)) {
		return "Stien m ikke indeholde &#034;" . $entity_map[$match[1]] . "&#034;";
	    }
	}
    } elseif ($type == "xs:anyType") {
	$parser = xml_parser_create();
	if (!xml_parse($parser, $xml_header . "<root>$value</root>")) {
	    xml_parser_free($parser);
	    return "Indholdet er ikke XHTML validt";
	}
	xml_parser_free($parser);
    }
	

    return 0;
}


function complextype_valid ($node, &$namespace, $level, &$path) {
    global $debug;

    if ($debug) {
	echo "Validating complexType<br>";
    }
}


function simpletype_valid ($node, &$namespace, $level, &$path) {
    global $debug;

    if ($debug) {
	echo "Validating simpleType<br>";
    }
}

// $Log: xform_valid.php,v $
// Revision 1.6  2002/09/12 19:15:38  sondberg
// No < > in fields other than xs:anyType.
//
// Revision 1.5  2002/09/12 14:12:10  sondberg
// Now it checks for well-formed ness, before saving.
//
// Revision 1.4  2002/09/12 13:08:45  sondberg
// *** empty log message ***
//
// Revision 1.3  2002/07/04 11:54:43  sondberg
// Added validation patterns for xs:anyURI.
//
// Revision 1.2  2002/06/11 12:16:17  sondberg
// Implemented the first version of element_valid.
//
// Revision 1.1  2002/06/07 12:57:20  sondberg
// Added this file to repository.
//

?>
